/*
Created: 06/09/2010
*/

#include <LinearMath/btAlignedObjectArray.h>
#include <BulletCollision/CollisionShapes/btScaledBvhTriangleMeshShape.h>
#include <BulletCollision/CollisionShapes/btUniformScalingShape.h>
#include <BulletCollision/CollisionShapes/btScaledBvhTriangleMeshShape.h>
#include <BulletCollision/CollisionShapes/btCompoundShape.h>
#include "btCollisionShapeGarbageCollector.h"


//================================================================
// class btCollisionShapeGarbageCollector
//================================================================
void btCollisionShapeGarbageCollector::cleanList()	{
	removeDoublesAndNulls();
	flattenHierarchy(true);
}
void btCollisionShapeGarbageCollector::removeDoublesAndNulls()	{
		for (int i=0;i<size()-1;i++)	{
			btCollisionShape* item = operator[](i);
			if (!item) {
				Base::swap(i,size()-1);
				Base::pop_back();
				--i;
				continue;
			}
			for (int j=i+1;j<size();j++)	{
				if (item == operator[](j)) {
					Base::swap(j,size()-1);
					Base::pop_back();
					--j;
				}
			}
		}
}
void btCollisionShapeGarbageCollector::flattenHierarchy(bool dontInsertDoublesAndNulls)	{
	const int parentShapeArraySize = size();
	for (int i = 0; i < parentShapeArraySize;i++)	{
		addChildShapesOf(operator[](i),dontInsertDoublesAndNulls,false);
	}
}
void btCollisionShapeGarbageCollector::addChildShapesOf(btCollisionShape* shape,bool dontInsertDoublesAndNulls,bool addThePassedShapeToo)	{
		if (addThePassedShapeToo) push_back(shape,dontInsertDoublesAndNulls);
		if (!shape) return;
		switch (shape->getShapeType())	{
			case COMPOUND_SHAPE_PROXYTYPE:
				{
					btCompoundShape* cs = static_cast <btCompoundShape*> (shape);
					for (int j=0,childSz=cs->getNumChildShapes();j<childSz;j++)	{
						addChildShapesOf(cs->getChildShape(j),dontInsertDoublesAndNulls,true);
					}
				}
			break;
			case UNIFORM_SCALING_SHAPE_PROXYTYPE:
				{
					btUniformScalingShape* s = static_cast <btUniformScalingShape*> (shape);
					addChildShapesOf(s->getChildShape(),dontInsertDoublesAndNulls,true);
				}
			break;
			case SCALED_TRIANGLE_MESH_SHAPE_PROXYTYPE:
				{
					btScaledBvhTriangleMeshShape* s = static_cast <btScaledBvhTriangleMeshShape*> (shape);
					addChildShapesOf(s->getChildShape(),dontInsertDoublesAndNulls,true);
				}	
			break;
		}
}
void btCollisionShapeGarbageCollector::deleteAll()	{		
		cleanList();
		for (int i=size()-1;i>=0;i--)	{
			btCollisionShape* s = operator[](i);
			if (s) {
				if (deleteUserPointerCallback)  (*deleteUserPointerCallback)(*s);
				try	{
					// We could delete something related to the user pointer here...
					delete s;s=NULL;
				}
				catch(...)	{
				}	
			}
		}
		Base::clear();
	}
//=================================================================

